import { Accessor } from 'solid-js';

/**
 * Creates a regex from a search string. This regex can be used to match any of the words in the search string.
 *
 * @param search The search string to create a regex from.
 * @returns A regex that matches any of the words in the search string.
 */
declare function makeSearchRegex(search: string): RegExp;
/**
 * Creates a function for marking parts of a string that match a regex. Each match will be mapped by {@link mapMatch} and returned as an array of strings and mapped values.
 *
 * @see https://github.com/solidjs-community/solid-primitives/tree/main/packages/marker#createMarker
 *
 * @param mapMatch A callback for mapping the matched string to a value to be added to the returned array.
 * @param options Options for the marker.
 * - `cacheSize` The maximum number of elements to cache for reuse. Defaults to 100.
 *
 * @returns A function that takes a string input and a regex to match the input, and returns an array of strings and mapped values.
 *
 * @example
 * ```tsx
 * const highlight = createMarker((text) => <mark>{text()}</mark>);
 *
 * <p>
 *   {highlight("Hello world!", /\w+/g)} // => <mark>Hello</mark> <mark>world</mark>!
 * </p>
 * ```
 */
declare function createMarker<T>(mapMatch: (match: Accessor<string>) => T, options?: {
    cacheSize?: number;
}): (string: string, regex: RegExp) => (string | T)[];

export { createMarker, makeSearchRegex };
